@@ -7,7 +7,7 @@ from debmutate.control import (
     is_relation_implied,
     )
 from lintian_brush.fixer import control, report_result, LintianIssue
-from lintian_brush.lintian import read_debhelper_lintian_data_file
+from lintian_brush.lintian import read_debhelper_lintian_data_file, LINTIAN_DATA_PATH
 from debmutate._rules import Makefile, Rule, dh_invoke_get_with
 import os
 import shlex
@@ -16,14 +16,14 @@ import sys
 COMMAND_TO_DEP = {}
 
 
-if not os.path.isdir('/usr/share/lintian/data'):
+if not os.path.isdir(LINTIAN_DATA_PATH):
     # lintian doesn't appear to be installed
     sys.exit(2)
 
 
 for path, sep in [
-    ('/usr/share/lintian/data/debhelper/dh_commands', '='),
-    ('/usr/share/lintian/data/debhelper/dh_commands-manual', '||'),
+    (os.path.join(LINTIAN_DATA_PATH, 'debhelper/dh_commands'), '='),
+    (os.path.join(LINTIAN_DATA_PATH, 'debhelper/dh_commands-manual'), '||'),
         ]:
     with open(path, 'r') as f:
         COMMAND_TO_DEP.update(read_debhelper_lintian_data_file(f, sep))
@@ -31,18 +31,45 @@ for path, sep in [
 
 ADDON_TO_DEP = {}
 
+with open(os.path.join(LINTIAN_DATA_PATH, 'common/dh_addons'), 'r') as f:
+    ADDON_TO_DEP.update(read_debhelper_lintian_data_file(f, '='))
 
-for path, sep in [
-    ('/usr/share/lintian/data/common/dh_addons', '='),
-    ('/usr/share/lintian/data/debhelper/dh_addons-manual', '||'),
-        ]:
-    with open(path, 'r') as f:
-        ADDON_TO_DEP.update(read_debhelper_lintian_data_file(f, sep))
-
+ADDON_TO_DEP.update({
+    # Copied from /usr/share/lintian/lib/Lintian/Check/Debhelper.pm
+    'ada_library': 'dh-ada-library | dh-sequence-ada-library',
+    'apache2': 'dh-apache2 | apache2-dev',
+    'autoreconf':
+        'dh-autoreconf | debhelper (>= 9.20160403~) | debhelper-compat',
+    'cli': 'cli-common-dev | dh-sequence-cli',
+    'dwz': 'debhelper | debhelper-compat | dh-sequence-dwz',
+    'installinitramfs':
+        'debhelper | debhelper-compat | dh-sequence-installinitramfs',
+    'gnome': 'gnome-pkg-tools | dh-sequence-gnome',
+    'lv2config': 'lv2core',
+    'nodejs': 'pkg-js-tools | dh-sequence-nodejs',
+    'perl_dbi': 'libdbi-perl | dh-sequence-perl-dbi',
+    'perl_imager': 'libimager-perl | dh-sequence-perl-imager',
+    'pgxs': 'postgresql-server-dev-all | postgresql-all',
+    'pgxs_loop':  'postgresql-server-dev-all | postgresql-all',
+    'pypy': 'dh-python | dh-sequence-pypy',
+    'python2': 'python2:any | python2-dev:any | dh-sequence-python2',
+    'python3':
+        'python3:any | python3-all:any | python3-dev:any | '
+        'python3-all-dev:any | dh-sequence-python3',
+    'scour':  'scour | python-scour | dh-sequence-scour',
+    'sphinxdoc':
+        'sphinx | python-sphinx | python3-sphinx | dh-sequence-sphinxdoc',
+    'systemd':
+        'debhelper (>= 9.20160709~) | debhelper-compat | '
+        'dh-sequence-systemd | dh-systemd',
+})
 
 need = []
 
-mf = Makefile.from_path('debian/rules')
+try:
+    mf = Makefile.from_path('debian/rules')
+except FileNotFoundError:
+    sys.exit(0)
 
 for entry in mf.contents:
     if isinstance(entry, Rule):
