@@ -24,6 +24,7 @@ from lintian_brush.fixer import (
     compat_release,
     control,
     current_package_version,
+    is_debcargo_package,
     fixed_lintian_tag,
     report_result,
     warn,
@@ -134,7 +135,7 @@ else:
     except FileNotFoundError:
         # debcargo just uses the latest version and doesn't store debhelper
         # version explicitly.
-        if os.path.exists('debian/debcargo.toml'):
+        if is_debcargo_package():
             sys.exit(0)
         raise
 
@@ -216,6 +217,8 @@ class PybuildUpgrader(object):
                 buildsystem = detect_debhelper_buildsystem(step)
                 if buildsystem == 'python_distutils':
                     line += b' --buildsystem=pybuild'
+                    subitems.add(
+                        'Replace python_distutils buildsystem with pybuild.')
                     self.upgraded = True
             else:
                 if b'buildsystem=pybuild' in line:
@@ -228,6 +231,7 @@ class PybuildUpgrader(object):
             line, rest = line.split(b' -- ', 1)
             if step is None:
                 step = line[len(b'dh_auto_'):].split(b' ', 1)[0].decode()
+            subitems.add('Replace python_distutils buildsystem with pybuild.')
             line = (b'PYBUILD_' + step.upper().encode() + b'_ARGS=' +
                     shlex.quote(rest.decode()).encode() + b' ' + line)
 
@@ -329,6 +333,18 @@ def debhelper_argument_order(line, target):
     return line
 
 
+def override_dh_auto_test_drop_options(rule):
+    if b'override_dh_auto_test' not in rule.targets:
+        return
+    if rule.lines[1] != b'ifeq (,$(filter nocheck,$(DEB_BUILD_OPTIONS)))' or rule.lines[-1] != b'endif':
+        return
+    subitems.add(
+        'Drop check for DEB_BUILD_OPTIONS containing "nocheck", '
+        'since debhelper now does this.')
+    del rule.lines[1]
+    del rule.lines[-1]
+
+
 def upgrade_to_debhelper_12():
 
     pybuild_upgrader = PybuildUpgrader()
@@ -454,7 +470,7 @@ def upgrade_to_debhelper_13():
         # be reverted to a non-fatal warning by explicitly passing
         # --list-missing like it was in compat 12.
         drop_dh_missing_fail,
-        ])
+        ], rule_cb=override_dh_auto_test_drop_options)
 
 
 upgrade_to_debhelper = {
