@@ -3,15 +3,15 @@
 import asyncio
 import re
 import socket
-import sys
 from lintian_brush import USER_AGENT, DEFAULT_URLLIB_TIMEOUT
-from debmutate.control import ControlEditor
 from debmutate.vcs import (
     get_vcs_info,
     )
 from lintian_brush.fixer import (
+    control,
     net_access_allowed,
     fixed_lintian_tag,
+    warn,
     report_result,
     )
 from lintian_brush.salsa import (
@@ -97,7 +97,7 @@ def find_new_urls(vcs_type, vcs_url, package, maintainer_email,
             (vcs_type, vcs_url, vcs_browser) = loop.run_until_complete(
                 retrieve_vcswatch_urls(package))
         except VcsWatchError as e:
-            sys.stderr.write('vcswatch URL unusable: %s\n' % e.args[0])
+            warn('vcswatch URL unusable: %s' % e.args[0])
         except KeyError:
             pass
         else:
@@ -108,8 +108,7 @@ def find_new_urls(vcs_type, vcs_url, package, maintainer_email,
                         determine_browser_url(vcs_type, vcs_url) or
                         vcs_browser)
                 return (vcs_type, vcs_url, vcs_browser)
-            sys.stderr.write(
-                'vcswatch URL %s is still on old infrastructure.' % vcs_url)
+            warn('vcswatch URL %s is still on old infrastructure.' % vcs_url)
 
     # Otherwise, attempt to guess based on maintainer email.
     guessed_url = guess_repository_url(package, maintainer_email)
@@ -182,7 +181,7 @@ def migrate_from_obsolete_infra(control):
             pass
 
 
-with ControlEditor() as updater:
+with control as updater:
     migrate_from_obsolete_infra(updater.source)
 
 report_result()
