@@ -32,9 +32,9 @@ import time
 import traceback
 from typing import Optional, List, Sequence, Iterator, Iterable, Tuple
 
-from debian.changelog import Changelog, Version
+from tqdm import trange
 
-from breezy import ui
+from debian.changelog import Changelog, Version
 
 import breezy.bzr  # noqa: F401
 import breezy.git  # noqa: F401
@@ -52,7 +52,7 @@ from debian.deb822 import Deb822
 from debmutate.reformatting import FormattingUnpreservable
 
 
-__version__ = (0, 107)
+__version__ = (0, 114)
 version_string = ".".join(map(str, __version__))
 SUPPORTED_CERTAINTIES = ["certain", "confident", "likely", "possible", None]
 DEFAULT_MINIMUM_CERTAINTY = "certain"
@@ -369,6 +369,9 @@ class PythonScriptFixer(Fixer):
     def __repr__(self):
         return "<%s(%r)>" % (self.__class__.__name__, self.name)
 
+    def __str__(self):
+        return self.name
+
     def run(
         self,
         basedir,
@@ -452,6 +455,9 @@ class ScriptFixer(Fixer):
     def __repr__(self):
         return "<ScriptFixer(%r)>" % self.name
 
+    def __str__(self):
+        return self.name
+
     def run(
         self,
         basedir: str,
@@ -513,6 +519,10 @@ def find_fixers_dir() -> str:
     if os.path.isdir(resource_dir):
         return resource_dir
     # Urgh.
+    local_share_dir = "/usr/local/share/lintian-brush/fixers"
+    if os.path.isdir(local_share_dir):
+        return local_share_dir
+
     return "/usr/share/lintian-brush/fixers"
 
 
@@ -1164,13 +1174,10 @@ def run_lintian_fixers(
         local_tree, subpath=subpath, use_inotify=use_inotify
     )
     ret = ManyResult()
-    with ui.ui_factory.nested_progress_bar() as pb:
+    with trange(len(fixers), leave=False, disable=None) as t:
         for i, fixer in enumerate(fixers):
-            pb.update(
-                "Running fixer %r on %s" % (fixer, local_tree.abspath(subpath)),
-                i,
-                len(fixers),
-            )
+            t.set_description("Running fixer %s" % fixer)
+            t.update()
             start = time.time()
             if dirty_tracker:
                 dirty_tracker.mark_clean()
@@ -1260,7 +1267,7 @@ def confidence_to_certainty(confidence: Optional[int]) -> str:
 
 def min_certainty(certainties: Sequence[str]) -> str:
     return confidence_to_certainty(
-        max([certainty_to_confidence(c) for c in certainties] + [0])
+        max([SUPPORTED_CERTAINTIES.index(c) for c in certainties] + [0])
     )
 
 
