@@ -17,18 +17,21 @@
 
 import argparse
 import atexit
+import json
 import logging
+import os
 import shutil
 import sys
 import tempfile
+from typing import Set, Optional
 
-from debian.changelog import get_maintainer
+from debian.changelog import get_maintainer, ChangelogCreateError
 import distro_info
 
+import breezy
 from breezy.branch import Branch
 from breezy.workingtree import WorkingTree
 
-import breezy  # noqa: E402
 from breezy.errors import (  # noqa: E402
     DependencyNotPresent,
     NotBranchError,
@@ -41,6 +44,7 @@ import breezy.bzr  # noqa: E402
 from . import (  # noqa: E402
     NotDebianPackage,
     PendingChanges,
+    DescriptionMissing,
     available_lintian_fixers,
     find_fixers_dir,
     get_committer,
@@ -49,10 +53,51 @@ from . import (  # noqa: E402
     version_string,
     SUPPORTED_CERTAINTIES,
     DEFAULT_MINIMUM_CERTAINTY,
+    control_files_in_root,
+    version_string as lintian_brush_version_string,
 )
 from .config import Config  # noqa: E402
 
 
+DEFAULT_ADDON_FIXERS = [
+    "debian-changelog-line-too-long",
+    "file-contains-trailing-whitespace",
+    "out-of-date-standards-version",
+    "package-uses-old-debhelper-compat-version",
+    "public-upstream-key-not-minimal",
+    "no-dh-sequencer",
+]
+
+DEFAULT_VALUE_LINTIAN_BRUSH_ADDON_ONLY = 10
+DEFAULT_VALUE_LINTIAN_BRUSH = 50
+# Base these scores on the importance as set in Debian?
+LINTIAN_BRUSH_TAG_VALUES = {
+    "file-contains-trailing-whitespace": 0,
+}
+LINTIAN_BRUSH_TAG_DEFAULT_VALUE = 5
+
+
+def calculate_value(tags: Set[str]) -> int:
+    if not (set(tags) - set(DEFAULT_ADDON_FIXERS)):
+        value = DEFAULT_VALUE_LINTIAN_BRUSH_ADDON_ONLY
+    else:
+        value = DEFAULT_VALUE_LINTIAN_BRUSH
+    for tag in tags:
+        value += LINTIAN_BRUSH_TAG_VALUES.get(tag, LINTIAN_BRUSH_TAG_DEFAULT_VALUE)
+    return value
+
+
+def report_fatal(code: str, description: str, hint: Optional[str] = None) -> None:
+    if os.environ.get('SVP_API') == '1':
+        with open(os.environ['SVP_RESULT'], 'w') as f:
+            json.dump({
+                'result_code': code,
+                'description': description}, f)
+    logging.fatal('%s', description)
+    if hint:
+        logging.info('%s', hint)
+
+
 def main(argv=None):  # noqa: C901
     parser = argparse.ArgumentParser(prog="lintian-brush")
 
@@ -113,7 +158,7 @@ def main(argv=None):  # noqa: C901
         "--allow-reformatting",
         default=None,
         action="store_true",
-        help=argparse.SUPPRESS,
+        help="Allow file reformatting and stripping of comments."
     )
     package_group.add_argument(
         "--no-update-changelog",
@@ -133,8 +178,8 @@ def main(argv=None):  # noqa: C901
 
     output_group = parser.add_argument_group("output")
     output_group.add_argument(
-        "--verbose", help="be verbose", action="store_true", default=False
-    )
+        "--verbose", help="be verbose", action="store_true",
+        default=('SVP_API' in os.environ))
     output_group.add_argument(
         "--diff", help="Print resulting diff afterwards.", action="store_true"
     )
@@ -195,13 +240,13 @@ def main(argv=None):  # noqa: C901
     )
     if args.list_fixers:
         for script in sorted([fixer.name for fixer in fixers]):
-            print('%s', script)
+            print(script)
     elif args.list_tags:
         tags = set()
         for fixer in fixers:
             tags.update(fixer.lintian_tags)
         for tag in sorted(tags):
-            print('%s', tag)
+            print(tag)
     else:
         try:
             if args.dry_run:
@@ -274,6 +319,12 @@ def main(argv=None):  # noqa: C901
         if allow_reformatting is None:
             allow_reformatting = False
         with wt.lock_write():
+            if control_files_in_root(wt, subpath):
+                report_fatal(
+                    "control-files-in-root",
+                    "control files live in root rather than debian/ " "(LarstIQ mode)",
+                )
+
             try:
                 overall_result = run_lintian_fixers(
                     wt,
@@ -291,7 +342,7 @@ def main(argv=None):  # noqa: C901
                     diligence=args.diligence,
                 )
             except NotDebianPackage:
-                logging.error("%s: Not a debian package.", wt.basedir)
+                report_fatal("not-debian-package", "Not a Debian package")
                 return 1
             except PendingChanges:
                 logging.error("%s: Please commit pending changes first.", wt.basedir)
@@ -300,6 +351,18 @@ def main(argv=None):  # noqa: C901
 
                     show_tree_status(wt)
                 return 1
+            except ChangelogCreateError as e:
+                report_fatal(
+                    "changelog-create-error", "Error creating changelog entry: %s" % e
+                )
+                return 1
+            except DescriptionMissing as e:
+                report_fatal(
+                    "fixer-description-missing",
+                    "Fixer %s made changes but did not provide description." %
+                    e.fixer)
+                return 1
+
         if overall_result.overridden_lintian_issues:
             if len(overall_result.overridden_lintian_issues) == 1:
                 logging.info(
@@ -314,10 +377,11 @@ def main(argv=None):  # noqa: C901
             for result, summary in overall_result.success:
                 all_tags.update(result.fixed_lintian_tags)
             if all_tags:
-                logging.info("Lintian tags fixed: %r" % all_tags)
+                logging.info("Lintian tags fixed: %r", all_tags)
             else:
                 logging.info(
-                    "Some changes were made, " "but there are no affected lintian tags."
+                    "Some changes were made, "
+                    "but there are no affected lintian tags."
                 )
             min_certainty = overall_result.minimum_success_certainty()
             if min_certainty != "certain":
@@ -327,7 +391,8 @@ def main(argv=None):  # noqa: C901
                     min_certainty,
                 )
         else:
-            logging.info("No changes made.")
+            report_fatal("nothing-to-do", "No changes made.")
+            return 0
         if overall_result.failed_fixers and not args.verbose:
             logging.info(
                 "Some fixer scripts failed to run: %r. "
@@ -347,6 +412,39 @@ def main(argv=None):  # noqa: C901
             show_diff_trees(
                 wt.branch.repository.revision_tree(since_revid), wt, sys.stdout.buffer
             )
+        if os.environ.get('SVP_API') == '1':
+            applied = []
+            if 'SVP_RESUME' in os.environ:
+                with open(os.environ['SVP_RESUME'], 'r') as f:
+                    base = json.load(f)
+                    applied.extend(base['applied'])
+            all_fixed_lintian_tags = set()
+            for result, summary in overall_result.success:
+                applied.append(
+                    {
+                        "summary": summary,
+                        "description": result.description,
+                        "fixed_lintian_tags": result.fixed_lintian_tags,
+                        "revision_id": result.revision_id.decode("utf-8"),
+                        "certainty": result.certainty,
+                    }
+                )
+                all_fixed_lintian_tags.update(result.fixed_lintian_tags)
+            failed = {
+                name: str(e)
+                for (name, e) in overall_result.failed_fixers.items()}
+            with open(os.environ['SVP_RESULT'], 'w') as f:
+                json.dump({
+                    'value': calculate_value(all_fixed_lintian_tags),
+                    'context': {
+                        'applied': applied,
+                        'failed': failed,
+                        "versions": {
+                            "lintian-brush": lintian_brush_version_string,
+                            "breezy": breezy.version_string,
+                            }
+                        }
+                    }, f)
 
 
 if __name__ == "__main__":
