@@ -18,12 +18,15 @@
 
 """Debhelper utility functions."""
 
+from functools import cache
+import json
 import os
 import subprocess
-from typing import Dict, Optional
-
-from .lintian import read_debhelper_lintian_data_file, LINTIAN_DATA_PATH
+from typing import Dict, Optional, List, Callable
 
+from debian.changelog import Version
+from debmutate.control import drop_dependency, add_dependency
+from debmutate._rules import update_rules, dh_invoke_drop_with, dh_invoke_add_with
 
 DEBHELPER_BUILD_STEPS = ["configure", "build", "test", "install", "clean"]
 
@@ -46,7 +49,7 @@ def detect_debhelper_buildsystem(step: Optional[str] = None) -> Optional[str]:
             "-MDebian::Debhelper::Dh_Buildsystems",
             "-e",
             """\
-Debian::Debhelper::Dh_Lib::init();
+Debian::Debhelper::Dh_Lib::init(inhibit_log=>1);
 my $b=Debian::Debhelper::Dh_Buildsystems::load_buildsystem(undef, %(step)s);\
 if (defined($b)) { print($b->NAME); } else { print("_undefined_"); }\
 """
@@ -58,29 +61,21 @@ if (defined($b)) { print($b->NAME); } else { print("_undefined_"); }\
     return output
 
 
-LINTIAN_COMPAT_LEVEL_PATH = os.path.join(LINTIAN_DATA_PATH, "debhelper/compat-level")
-
-
+@cache
 def _get_lintian_compat_levels() -> Dict[str, int]:
-    with open(LINTIAN_COMPAT_LEVEL_PATH, "r") as f:
-        return {
-            key: int(value) for (key, value) in read_debhelper_lintian_data_file(f, "=")
-        }
+    output = subprocess.check_output(
+        ["dh_assistant", "supported-compat-levels"]).decode()
+    return json.loads(output)
 
 
 def lowest_non_deprecated_compat_level() -> int:
     """Find the lowest non-deprecated debhelper compat level."""
-    return _get_lintian_compat_levels()["deprecated"]
+    return _get_lintian_compat_levels()["LOWEST_NON_DEPRECATED_COMPAT_LEVEL"]
 
 
 def highest_stable_compat_level() -> int:
     """Find the highest stable debhelper compat level."""
-    return _get_lintian_compat_levels()["recommended"]
-
-
-def pedantic_compat_level() -> int:
-    """Find the highest stable debhelper compat level."""
-    return _get_lintian_compat_levels()["pedantic"]
+    return _get_lintian_compat_levels()["HIGHEST_STABLE_COMPAT_LEVEL"]
 
 
 def maximum_debhelper_compat_version(compat_release: str) -> int:
@@ -101,7 +96,10 @@ def maximum_debhelper_compat_version(compat_release: str) -> int:
     return max_version
 
 
-def write_rules_template(path, buildsystem=None, addons=None, env=None):
+def write_rules_template(
+        path: str, buildsystem: Optional[str] = None,
+        addons: Optional[List[str]] = None,
+        env: Optional[Dict[str, str]] = None) -> None:
     if addons is None:
         addons = []
     dh_args = ["$@"]
@@ -131,3 +129,60 @@ def write_rules_template(path, buildsystem=None, addons=None, env=None):
 """
         )
     os.chmod(path, 0o755)
+
+
+def drop_obsolete_maintscript_entries(
+        editor, should_remove: Callable[[str, Version], bool]) -> List[int]:
+    """Drop obsolete entries from a maintscript file.
+
+    Args:
+      editor: editor to use to access the maintscript
+      should_remove: callable to check whether a package/version tuple is obsolete
+    Returns:
+      list of indexes of entries that were removed
+    """
+    remove = []
+    comments = []
+    entries_removed = []
+    for i, entry in enumerate(list(editor.lines)):
+        if isinstance(entry, str):
+            comments.append(i)
+            continue
+        prior_version = getattr(entry, "prior_version", None)
+        if prior_version is not None:
+            if should_remove(entry.package, Version(prior_version)):
+                remove.extend(comments)
+                remove.append(i)
+                entries_removed.append(i)
+        comments = []
+    removed = []
+    for i in reversed(remove):
+        removed.append(editor.lines[i])
+        del editor.lines[i]
+    return entries_removed
+
+
+def drop_sequence(control, rules, sequence):
+    new_depends = drop_dependency(
+        control.source.get("Build-Depends", ""), "dh-" + sequence)
+    if new_depends != control.source['Build-Depends']:
+        def drop_with(line, target):
+            return dh_invoke_drop_with(line, sequence.replace('-', '_').encode())
+        update_rules(drop_with)
+    new_depends = drop_dependency(
+        new_depends, "dh-sequence-" + sequence)
+    if control['Build-Depends'] == new_depends:
+        return False
+    control['Build-Depends'] = new_depends
+    return True
+
+
+def add_sequence(control, rules, sequence):
+    control.source['Build-Depends'] = add_dependency(
+        control.source.get('Build-Depends'), 'dh-vim-addon')
+
+    def add_with(line, target):
+        if line.startswith(b'dh ') or line.startswith(b'dh_'):
+            return dh_invoke_add_with(line, sequence.replace('-', '_').encode())
+        return line
+    update_rules(add_with)
