@@ -24,7 +24,7 @@ LINTIAN_DATA_PATH = '/usr/share/lintian/data'
 
 
 def read_debhelper_lintian_data_file(f, sep):
-    for lineno, line in enumerate(f, 1):
+    for line in f:
         if line.startswith("#"):
             continue
         if not line.strip():
@@ -91,7 +91,8 @@ def _read_test_fields(path, vendor):
 
 
 def known_tests_control_fields(vendor):
-    return _read_test_fields(KNOWN_TESTS_CONTROL_FIELDS_PATH, vendor)
+    return set(['Tests', 'Restrictions', 'Features', 'Depends',
+                'Tests-Directory', 'Test-Command'])
 
 
 KNOWN_SOURCE_FIELDS_PATH = os.path.join(
@@ -108,3 +109,12 @@ KNOWN_BINARY_FIELDS_PATH = os.path.join(
 
 def known_binary_fields(vendor):
     return _read_test_fields(KNOWN_BINARY_FIELDS_PATH, vendor)
+
+
+DEBHELPER_DH_COMMANDS_PATH = os.path.join(
+    LINTIAN_DATA_PATH, 'debhelper/dh_commands')
+
+
+def dh_commands():
+    with open(DEBHELPER_DH_COMMANDS_PATH, 'r') as f:
+        return list(read_debhelper_lintian_data_file(f, '='))
