@@ -19,15 +19,21 @@
 """Utility for dropping unnecessary constraints."""
 
 import asyncio
+import json
 import logging
 import os
+from typing import List, Tuple, Optional, Dict, Callable
 
-from breezy.commit import PointlessCommit
+from breezy.commit import PointlessCommit, NullCommitReporter
+from breezy.workingtree import WorkingTree
 
+from debmutate.control import PkgRelation
+from debmutate.deb822 import ChangeConflict
 from debmutate.debhelper import MaintscriptEditor
-
+from debmutate.reformatting import FormattingUnpreservable
 
 from debian.changelog import Version
+from debian.deb822 import Deb822Dict
 
 from debmutate.control import (
     ControlEditor,
@@ -42,10 +48,19 @@ from debmutate.reformatting import (
 )
 
 from .changelog import add_changelog_entry
-from . import get_committer, control_files_in_root, NotDebianPackage
+from . import (
+    get_committer,
+    control_files_in_root,
+    control_file_present,
+    NotDebianPackage,
+    is_debcargo_package,
+    )
 from .debhelper import drop_obsolete_maintscript_entries
 
 
+DEFAULT_VALUE_MULTIARCH_HINT = 30
+
+
 _changelog_policy_noted = False
 
 
@@ -60,7 +75,7 @@ def _note_changelog_policy(policy, msg):
     _changelog_policy_noted = True
 
 
-def depends_obsolete(latest_version, kind, req_version):
+def depends_obsolete(latest_version: Version, kind: str, req_version: Version):
     req_version = Version(req_version)
     if kind == ">=":
         return latest_version >= req_version
@@ -71,7 +86,7 @@ def depends_obsolete(latest_version, kind, req_version):
     return False
 
 
-def conflict_obsolete(latest_version, kind, req_version):
+def conflict_obsolete(latest_version: Version, kind: str, req_version: Version):
     req_version = Version(req_version)
     if kind == "<<":
         return latest_version >= req_version
@@ -80,13 +95,13 @@ def conflict_obsolete(latest_version, kind, req_version):
     return False
 
 
-async def _package_version(source, release):
+async def _package_version(package: str, release: str) -> Optional[Version]:
     from .udd import connect_udd_mirror
 
     conn = await connect_udd_mirror()
     version = await conn.fetchval(
-        "select version from sources where source = $1 and release = $2",
-        source,
+        "select version from packages where package = $1 and release = $2",
+        package,
         release,
     )
     if version is not None:
@@ -94,33 +109,95 @@ async def _package_version(source, release):
     return None
 
 
-def package_version(source, release):
-    loop = asyncio.get_event_loop()
-    return loop.run_until_complete(_package_version(source, release))
+async def _package_provides(package: str, release: str) -> Optional[Version]:
+    from .udd import connect_udd_mirror
+
+    conn = await connect_udd_mirror()
+    provides = await conn.fetchval(
+        "select provides from packages where package = $1 and release = $2",
+        package, release)
+    if provides is not None:
+        return [r[1] for r in parse_relations(provides)]
+    return None
 
 
-def drop_obsolete_depends(entry, upgrade_release):
+async def _package_essential(package: str, release: str) -> bool:
+    from .udd import connect_udd_mirror
+
+    conn = await connect_udd_mirror()
+    return await conn.fetchval(
+        "select (essential = 'yes') from packages where package = $1 and release = $2",
+        package, release)
+
+
+async def _package_build_essential(package: str, release: str) -> bool:
+    from .udd import connect_udd_mirror
+
+    conn = await connect_udd_mirror()
+    depends = await conn.fetchval(
+        "select depends from packages where package = $1 and release = $2",
+        'build-essential', release)
+
+    build_essential = set()
+    for ws1, rel, ws2 in parse_relations(depends):
+        build_essential.update([r.name for r in rel])
+    return package in build_essential
+
+
+class PackageChecker(object):
+
+    def __init__(self, release: str, build: bool):
+        self.release = release
+        self.build = build
+
+    def package_version(self, package: str) -> Optional[Version]:
+        loop = asyncio.get_event_loop()
+        return loop.run_until_complete(_package_version(package, self.release))
+
+    def package_provides(self, package):
+        loop = asyncio.get_event_loop()
+        return loop.run_until_complete(_package_provides(package, self.release))
+
+    def is_essential(self, package: str) -> bool:
+        loop = asyncio.get_event_loop()
+        if self.build:
+            return loop.run_until_complete(_package_build_essential(package, self.release))
+        else:
+            return loop.run_until_complete(_package_essential(package, self.release))
+
+
+def drop_obsolete_depends(entry: List[PkgRelation], checker: PackageChecker):
     ors = []
     dropped = []
     for pkgrel in entry:
+        newrel = pkgrel
         if pkgrel.version is not None:
-            logging.debug("Relation: %s", pkgrel)
-            compat_version = package_version(pkgrel.name, upgrade_release)
+            compat_version = checker.package_version(pkgrel.name)
+            logging.debug(
+                "Relation: %s. Upgrade release %s has %r ",
+                pkgrel, checker.release, compat_version)
             if compat_version is not None and depends_obsolete(
                 compat_version, *pkgrel.version
             ):
-                pkgrel.version = None
+                newrel = PkgRelation.parse(pkgrel.str())[0]
+                newrel.version = None
                 dropped.append(pkgrel)
-        ors.append(pkgrel)
+                # If the package is essential, we don't need to maintain a
+                # dependency on it.
+                if checker.is_essential(pkgrel.name):
+                    return [], entry
+        ors.append(newrel)
+    # TODO: if dropped: Check if any ors are implied by existing other
+    # dependencies
     return ors, dropped
 
 
-def drop_obsolete_conflicts(entry, upgrade_release):
+def drop_obsolete_conflicts(checker: PackageChecker, entry: List[PkgRelation]):
     ors = []
     dropped = []
     for pkgrel in entry:
         if pkgrel.version is not None:
-            compat_version = package_version(pkgrel.name, upgrade_release)
+            compat_version = checker.package_version(pkgrel.name)
             if compat_version is not None and conflict_obsolete(
                 compat_version, *pkgrel.version
             ):
@@ -130,20 +207,24 @@ def drop_obsolete_conflicts(entry, upgrade_release):
     return ors, dropped
 
 
-def update_depends(base, field, upgrade_release):
+def update_depends(base: Deb822Dict, field: str, checker: PackageChecker) -> List[str]:
     return filter_relations(
         base, field,
-        lambda oldrelation: drop_obsolete_depends(oldrelation, upgrade_release))
+        lambda oldrelation: drop_obsolete_depends(oldrelation, checker))
 
 
-def relations_empty(rels):
+def _relations_empty(rels):
     for ws1, rel, ws2 in rels:
         if rel:
             return False
     return True
 
 
-def filter_relations(base, field, cb):
+RelationsCallback = Callable[[List[PkgRelation]], Tuple[List[PkgRelation], List[PkgRelation]]]
+
+
+def filter_relations(base: Deb822Dict, field: str, cb: RelationsCallback) -> List[str]:
+    """Update a relations field."""
     try:
         old_contents = base[field]
     except KeyError:
@@ -156,7 +237,7 @@ def filter_relations(base, field, cb):
     for i, (ws1, oldrelation, ws2) in enumerate(oldrelations):
         relation, dropped = cb(oldrelation)
         changed.extend([d.name for d in dropped])
-        if relation:
+        if relation == oldrelation or relation:
             newrelations.append((ws1, relation, ws2))
         elif i == 0 and len(oldrelations) > 1:
             # If the first item is removed, then copy the spacing to the next
@@ -164,7 +245,7 @@ def filter_relations(base, field, cb):
             oldrelations[1] = (ws1, oldrelations[1][1], ws2)
 
     if changed:
-        if relations_empty(newrelations):
+        if _relations_empty(newrelations):
             del base[field]
         else:
             base[field] = format_relations(newrelations)
@@ -172,46 +253,48 @@ def filter_relations(base, field, cb):
     return []
 
 
-def update_conflicts(base, field, upgrade_release):
+def update_conflicts(base: Deb822Dict, field: str, checker: PackageChecker) -> List[str]:
     return filter_relations(
         base, field,
-        lambda oldrelation: drop_obsolete_conflicts(oldrelation, upgrade_release))
+        lambda oldrelation: drop_obsolete_conflicts(checker, oldrelation))
 
 
-def drop_old_source_relations(source, compat_release):
+def drop_old_source_relations(source, compat_release) -> List[Tuple[str, List[str], str]]:
+    checker = PackageChecker(compat_release, build=True)
     ret = []
     for field in [
         "Build-Depends",
         "Build-Depends-Indep",
         "Build-Depends-Arch",
     ]:
-        packages = update_depends(source, field, compat_release)
+        packages = update_depends(source, field, checker)
         if packages:
-            ret.append((field, packages))
+            ret.append((field, packages, compat_release))
     for field in ["Build-Conflicts", "Build-Conflicts-Indep", "Build-Conflicts-Arch"]:
-        packages = update_conflicts(source, field, compat_release)
+        packages = update_conflicts(source, field, checker)
         if packages:
-            ret.append((field, packages))
+            ret.append((field, packages, compat_release))
     return ret
 
 
-def drop_old_binary_relations(binary, upgrade_release):
+def drop_old_binary_relations(binary, upgrade_release: str) -> List[Tuple[str, List[str], str]]:
+    checker = PackageChecker(upgrade_release, build=False)
     ret = []
     for field in ["Depends", "Breaks", "Suggests", "Recommends", "Pre-Depends"]:
-        packages = update_depends(binary, field, upgrade_release)
+        packages = update_depends(binary, field, checker)
         if packages:
-            ret.append((field, packages))
+            ret.append((field, packages, upgrade_release))
 
     for field in ["Conflicts", "Replaces", "Breaks"]:
-        packages = update_conflicts(binary, field, upgrade_release)
+        packages = update_conflicts(binary, field, checker)
         if packages:
-            ret.append((field, packages))
+            ret.append((field, packages, upgrade_release))
 
     return ret
 
 
-def drop_old_relations(editor, compat_release, upgrade_release):
-    dropped = []
+def drop_old_relations(editor, compat_release: str, upgrade_release: str) -> List[Tuple[Optional[str], List[Tuple[str, List[str], str]]]]:
+    dropped: List[Tuple[Optional[str], List[Tuple[str, List[str], str]]]] = []
     source_dropped = []
     try:
         check_generated_file(editor.path)
@@ -235,14 +318,16 @@ def drop_old_relations(editor, compat_release, upgrade_release):
     return dropped
 
 
-def update_maintscripts(wt, subpath, package, upgrade_release, allow_reformatting=False):
+def update_maintscripts(
+        wt: WorkingTree, subpath: str, checker: PackageChecker, package: str,
+        allow_reformatting: bool = False):
     ret = []
     for entry in os.scandir(wt.abspath(os.path.join(subpath))):
         if not (entry.name == "maintscript" or entry.name.endswith(".maintscript")):
             continue
         with MaintscriptEditor(entry.path, allow_reformatting=allow_reformatting) as editor:
             def can_drop(p, v):
-                compat_version = package_version(p or package, upgrade_release)
+                compat_version = checker.package_version(p or package)
                 return compat_version is not None and compat_version > v
             removed = drop_obsolete_maintscript_entries(editor, can_drop)
             if removed:
@@ -250,14 +335,24 @@ def update_maintscripts(wt, subpath, package, upgrade_release, allow_reformattin
     return ret
 
 
-def name_list(packages):
-    if len(packages) == 1:
-        return packages[0]
+def name_list(packages: List[str]) -> str:
+    """Format a list of package names for use in prose.
+
+    Args:
+      packages: non-empty list of packages to format
+    Returns:
+      human-readable string
+    """
+    if not packages:
+        raise ValueError(packages)
     std = list(sorted(set(packages)))
+    if len(std) == 1:
+        return std[0]
     return ", ".join(std[:-1]) + " and " + std[-1]
 
 
 class ScrubObsoleteResult(object):
+
     def __init__(self, specific_files, maintscript_removed, control_removed):
         self.specific_files = specific_files
         self.maintscript_removed = maintscript_removed
@@ -266,31 +361,45 @@ class ScrubObsoleteResult(object):
     def __bool__(self):
         return bool(self.control_removed) or bool(self.maintscript_removed)
 
-    def itemized(self):
-        summary = []
+    def value(self) -> int:
+        value = DEFAULT_VALUE_MULTIARCH_HINT
         for para, changes in self.control_removed:
-            for field, packages in changes:
+            for field, packages, release in changes:
+                value += len(packages) * 2
+        for path, removed, release in self.maintscript_removed:
+            value += len(removed)
+        return value
+
+    def itemized(self) -> Dict[str, List[str]]:
+        summary: Dict[str, List[str]] = {}
+        for para, changes in self.control_removed:
+            for field, packages, release in changes:
                 if para:
-                    summary.append(
+                    summary.setdefault(release, []).append(
                         "%s: Drop versioned constraint on %s in %s."
                         % (para, name_list(packages), field)
                     )
                 else:
-                    summary.append(
+                    summary.setdefault(release, []).append(
                         "%s: Drop versioned constraint on %s."
                         % (field, name_list(packages))
                     )
         if self.maintscript_removed:
             total_entries = sum(
-                [len(entries) for name, entries in self.maintscript_removed])
-            summary.append(
+                [len(entries) for name, entries, release in self.maintscript_removed])
+            summary.setdefault(self.maintscript_removed[0][2], []).append(
                 "Remove %d maintscript entries from %d files." % (
                     total_entries, len(self.maintscript_removed))
             )
         return summary
 
 
-def _scrub_obsolete(wt, debian_path, compat_release, upgrade_release, allow_reformatting):
+def _scrub_obsolete(
+        wt: WorkingTree,
+        debian_path: str,
+        compat_release: str,
+        upgrade_release: str,
+        allow_reformatting: bool = True) -> ScrubObsoleteResult:
     specific_files = []
     control_path = os.path.join(debian_path, "control")
     try:
@@ -307,9 +416,11 @@ def _scrub_obsolete(wt, debian_path, compat_release, upgrade_release, allow_refo
             raise NotDebianPackage(wt, debian_path)
 
     maintscript_removed = []
-    for path, removed in update_maintscripts(wt, debian_path, package, upgrade_release, allow_reformatting):
+    for path, removed in update_maintscripts(
+            wt, debian_path, PackageChecker(upgrade_release, build=False), package,
+            allow_reformatting):
         if removed:
-            maintscript_removed.append((path, removed))
+            maintscript_removed.append((path, removed, upgrade_release))
             specific_files.append(path)
 
     return ScrubObsoleteResult(
@@ -319,16 +430,20 @@ def _scrub_obsolete(wt, debian_path, compat_release, upgrade_release, allow_refo
     )
 
 
-def scrub_obsolete(wt, subpath, compat_release, upgrade_release, update_changelog=None,
-                   allow_reformatting=False):
-    from breezy.commit import NullCommitReporter
+def scrub_obsolete(
+        wt: WorkingTree, subpath: str, compat_release: str, upgrade_release: str,
+        update_changelog=None,
+        allow_reformatting: bool = False) -> ScrubObsoleteResult:
+    """Scrub obsolete entries.
+    """
 
     if control_files_in_root(wt, subpath):
         debian_path = subpath
     else:
         debian_path = os.path.join(subpath, 'debian')
 
-    result = _scrub_obsolete(wt, debian_path, compat_release, upgrade_release, allow_reformatting)
+    result = _scrub_obsolete(
+        wt, debian_path, compat_release, upgrade_release, allow_reformatting)
 
     if not result:
         return result
@@ -354,26 +469,25 @@ def scrub_obsolete(wt, subpath, compat_release, upgrade_release, update_changelo
             update_changelog = True
 
     if update_changelog:
-        add_changelog_entry(
-            wt,
-            changelog_path,
-            ["Remove constraints unnecessary since %s:" % upgrade_release]
-            + ["+ " + line for line in summary],
-        )
+        lines = []
+        for release, entries in summary.items():
+            lines.append("Remove constraints unnecessary since %s:" % release)
+            lines.extend(["+ " + line for line in entries])
+        add_changelog_entry(wt, changelog_path, lines)
         specific_files.append(changelog_path)
 
-    message = "\n".join(
-        ["Remove constraints unnecessary since %s." % upgrade_release, ""]
-        + ["* " + line for line in summary]
-        + ["", "Changes-By: deb-scrub-obsolete"]
-    )
+    lines = []
+    for release, entries in summary.items():
+        lines.extend(["Remove constraints unnecessary since %s" % release, ""])
+        lines.extend(["* " + line for line in entries])
+    lines.extend(["", "Changes-By: deb-scrub-obsolete"])
 
     committer = get_committer(wt)
 
     try:
         wt.commit(
             specific_files=specific_files,
-            message=message,
+            message="\n".join(lines),
             allow_pointless=False,
             reporter=NullCommitReporter(),
             committer=committer,
@@ -384,9 +498,26 @@ def scrub_obsolete(wt, subpath, compat_release, upgrade_release, update_changelo
     return result
 
 
-def main():
+def report_fatal(code: str, description: str) -> None:
+    if os.environ.get('SVP_API') == '1':
+        with open(os.environ['SVP_RESULT'], 'w') as f:
+            json.dump({
+                'result_code': code,
+                'description': description}, f)
+    logging.fatal('%s', description)
+
+
+def report_okay(code: str, description: str) -> None:
+    if os.environ.get('SVP_API') == '1':
+        with open(os.environ['SVP_RESULT'], 'w') as f:
+            json.dump({
+                'result_code': code,
+                'description': description}, f)
+    logging.info('%s', description)
+
+
+def main():  # noqa: C901
     import argparse
-    from breezy.workingtree import WorkingTree
     import breezy  # noqa: E402
 
     breezy.initialize()
@@ -400,7 +531,7 @@ def main():
     )
     from .config import Config
 
-    parser = argparse.ArgumentParser(prog="drop-backwards-compat")
+    parser = argparse.ArgumentParser(prog="deb-scrub-obsolete")
     parser.add_argument(
         "--directory",
         metavar="DIRECTORY",
@@ -472,7 +603,7 @@ def main():
     if args.debug:
         logging.basicConfig(level=logging.DEBUG)
     else:
-        logging.basicConfig(level=logging.INFO)
+        logging.basicConfig(level=logging.INFO, format='%(message)s')
 
     update_changelog = args.update_changelog
     allow_reformatting = args.allow_reformatting
@@ -493,25 +624,74 @@ def main():
             compat_release = cfg.compat_release()
 
     if compat_release is None:
-        compat_release = debian_info.stable()
+        compat_release = debian_info.codename('oldstable')
 
-    logging.info(
-        "Removing run time constraints unnecessary since %s"
-        " and build time constraints unnecessary since %s",
-        upgrade_release, compat_release)
+    if upgrade_release != compat_release:
+        logging.info(
+            "Removing run time constraints unnecessary since %s"
+            " and build time constraints unnecessary since %s",
+            upgrade_release, compat_release)
+    else:
+        logging.info(
+            "Removing run time and build time constraints unnecessary "
+            "since %s", compat_release)
 
     if allow_reformatting is None:
         allow_reformatting = False
 
+    if is_debcargo_package(wt, subpath):
+        report_fatal("nothing-to-do", "Package uses debcargo")
+        return 1
+    elif not control_file_present(wt, subpath):
+        report_fatal("missing-control-file", "Unable to find debian/control")
+        return 1
+
     try:
-        scrub_obsolete(
+        result = scrub_obsolete(
             wt, subpath, compat_release, upgrade_release,
             update_changelog=args.update_changelog,
             allow_reformatting=allow_reformatting
         )
-    except NotDebianPackage:
-        logging.fatal('not a Debian package.')
+    except FormattingUnpreservable as e:
+        report_fatal(
+            "formatting-unpreservable",
+            "unable to preserve formatting while editing %s" % e.path,
+        )
         return 1
+    except GeneratedFile as e:
+        report_fatal(
+            "generated-file", "unable to edit generated file: %r" % e
+        )
+        return 1
+    except NotDebianPackage:
+        report_fatal('not-debian-package', 'Not a Debian package.')
+        return 1
+    except ChangeConflict as e:
+        report_fatal('change-conflict', 'Generated file changes conflict: %s' % e)
+        return 1
+
+    if not result:
+        report_okay("nothing-to-do", "no obsolete constraints")
+        return 0
+
+    if os.environ.get("SVP_API") == "1":
+        with open(os.environ["SVP_RESULT"], "w") as f:
+            json.dump({
+                "description": "Remove constraints unnecessary since %s."
+                % upgrade_release,
+                "value": result.value(),
+                "context": {
+                    "specific_files": result.specific_files,
+                    "maintscript_removed": result.maintscript_removed,
+                    "control_removed": result.control_removed,
+                }
+            }, f)
+
+    logging.info("Scrub obsolete settings.")
+    for release, lines in result.itemized().items():
+        for line in lines:
+            logging.info("* %s", line)
+
     return 0
 
 
