@@ -44,8 +44,8 @@ def candidates_from_setup_py(path, good_upstream_versions: Set[str], net_access=
 
         traceback.print_exc()
         return
-    project = result.get_name()
-    version = result.get_version()
+    project = result.get_name()  # type: ignore
+    version = result.get_version()  # type: ignore
     if not project:
         return
     current_version_filenames = None
@@ -112,6 +112,15 @@ def candidates_from_upstream_metadata(
                     parsed_url, good_upstream_versions, net_access=net_access
                 )
 
+        archive = code.get('Archive')
+        if archive == 'CRAN':
+            yield from guess_cran_watch_entry(code['Name'])
+
+
+def guess_cran_watch_entry(name):
+    w = Watch(r'https://cran.r-project.org/src/contrib/%s_([-\d.]*)\.tar\.gz' % name)
+    yield w, "cran", "likely"
+
 
 def guess_github_watch_entry(parsed_url, good_upstream_versions, net_access=False):
     from breezy.branch import Branch
@@ -186,3 +195,29 @@ def candidates_from_hackage(package, good_upstream_versions, net_access=False):
     matching_pattern = r".*/%s-(.*).tar.gz" % package
     w = Watch(download_url, matching_pattern)
     yield w, "hackage", "certain"
+
+
+def fix_old_github_patterns(updater):
+    for w in getattr(updater.watch_file, "entries", []):
+        parsed_url = urlparse(w.url)
+
+        # only applies to github.com
+        if parsed_url.netloc != "github.com":
+            continue
+
+        parts = parsed_url.path.strip('/').split('/')
+        if len(parts) >= 3 and parts[2] in ('tags', 'releases'):
+            pass
+        elif len(parts) >= 2 and parts[0] == '.*' and parts[1] == 'archive':
+            pass
+        else:
+            continue
+
+        parts = w.matching_pattern.split('/')
+        if len(parts) > 2 and parts[-2] == 'archive':
+            parts.insert(-1, 'refs/tags')
+        w.matching_pattern = '/'.join(parts)
+
+
+def fix_watch_issues(updater):
+    fix_old_github_patterns(updater)
